﻿using System;
using System.Linq;
using Core.Dtos;
using Core.Entities;
using Core.IRepository;
using Core.IServices;
using Core.Messages;
using Core.Queries;
using Library.Helpers;
using Library.Logging;
using System.ComponentModel;

namespace Services
{
    public class CrudBaseService<TId, TEntity, TDto, TQuery> : ReadBaseService<TId, TEntity, TDto, TQuery>, ICrudBaseService<TId, TEntity, TDto, TQuery>
        where TDto : BaseDto<TId>
        where TEntity : BaseEntity<TId>
        where TQuery : BaseQuery<TEntity>
    {        
        public CrudBaseService(IUnitOfWork unitOfWork, IBaseRepository<TId, TEntity> Repository, ILogger Logger)
            : base(unitOfWork, Repository, Logger)
        {
        }

        protected virtual void EditForeignRelations(TEntity entity, TDto dto)
        {
        }

        protected virtual TEntity MapDtoToEntity(TDto dto)
        {
            return AutoMapper.Mapper.Map<TDto, TEntity>(dto);
        }

        protected virtual TEntity MapDtoToEntity(TDto dto, TEntity entity)
        {
            return AutoMapper.Mapper.Map(dto, entity);
        }

        public virtual BaseResponse<TId> Add(BaseRequest<TDto> request)
        {
            BaseResponse<TId> response = new BaseResponse<TId>();
            try
            {
                TEntity entity = MapDtoToEntity(request.Value);
                EditForeignRelations(entity, request.Value);

                if (entity.Validate())
                {
                    Repository.Add(entity);
                    UnitOfWork.Commit();
                    response.Value = entity.IdValue;
                    response.Success = true;
                }
                else
                {
                    response.Success = false;
                    response.BrokenRules = entity.BrokenRules;
                }
            }
            catch (Exception ex)
            {
                response.Success = false;
                response.ErrorMessage = ex.ToString();
                Logger.Log(ex);
            }

            return response;
        }

        public virtual BaseResponse Edit(BaseRequest<TDto> request)
        {
            BaseResponse response = new BaseResponse();
            try
            {
                TDto dto = request.Value;
                TEntity entity = Repository.FindBy(dto.IdValue);
                MapDtoToEntity(request.Value, entity);
                EditForeignRelations(entity, request.Value);

                if (entity.Validate())
                {
                    Repository.Edit(entity);
                    UnitOfWork.Commit();
                    response.Success = true;
                }
                else
                {
                    response.Success = false;
                    response.BrokenRules = entity.BrokenRules;
                }
            }
            catch (Exception ex)
            {
                response.Success = false;
                response.ErrorMessage = ex.ToString();

                Logger.Log(ex);
            }

            return response;
        }

        public virtual BaseResponse Delete(string id)
        {
            BaseResponse response = new BaseResponse();
            try
            {
                TEntity entity = Repository.FindBy(ConvertToIdType(id));
                Repository.Delete(entity);
                UnitOfWork.Commit();

                response.Success = true;
            }
            catch (Exception ex)
            {
                response.Success = false;
                response.ErrorMessage = ex.ToString();
                Logger.Log(ex);
            }

            return response;
        }
    }
}
